@@ -51,6 +51,7 @@ import datetime
 import math
 import plistlib
 from struct import pack, unpack
+from struct import error as struct_error
 import sys
 import time
 
@@ -94,19 +95,33 @@ def readPlist(pathOrFile):
     except NotBinaryPlistException as e:
         try:
             pathOrFile.seek(0)
-            result = plistlib.readPlist(pathOrFile)
+            result = None
+            if hasattr(plistlib, 'loads'):
+                contents = None
+                if isinstance(pathOrFile, (six.binary_type, six.text_type)):
+                    with open(pathOrFile, 'rb') as f:
+                        contents = f.read()
+                else:
+                    contents = pathOrFile.read()
+                result = plistlib.loads(contents)
+            else:
+                result = plistlib.readPlist(pathOrFile)
             result = wrapDataObject(result, for_binary=True)
         except Exception as e:
             raise InvalidPlistException(e)
-    if didOpen:
-        pathOrFile.close()
+    finally:
+        if didOpen:
+            pathOrFile.close()
     return result
 
 def wrapDataObject(o, for_binary=False):
     if isinstance(o, Data) and not for_binary:
-        o = plistlib.Data(o)
-    elif isinstance(o, plistlib.Data) and for_binary:
-        o = Data(o.data)
+        v = sys.version_info
+        if not (v[0] >= 3 and v[1] >= 4):
+            o = plistlib.Data(o)
+    elif isinstance(o, (six.binary_type, plistlib.Data)) and for_binary:
+        if hasattr(o, 'data'):
+            o = Data(o.data)
     elif isinstance(o, tuple):
         o = wrapDataObject(list(o), for_binary)
         o = tuple(o)
@@ -121,7 +136,14 @@ def wrapDataObject(o, for_binary=False):
 def writePlist(rootObject, pathOrFile, binary=True):
     if not binary:
         rootObject = wrapDataObject(rootObject, binary)
-        return plistlib.writePlist(rootObject, pathOrFile)
+        if hasattr(plistlib, "dump"):
+            if isinstance(pathOrFile, (six.binary_type, six.text_type)):
+                with open(pathOrFile, 'wb') as f:
+                    return plistlib.dump(rootObject, f)
+            else:
+                return plistlib.dump(rootObject, pathOrFile)
+        else:
+            return plistlib.writePlist(rootObject, pathOrFile)
     else:
         didOpen = False
         if isinstance(pathOrFile, (six.binary_type, six.text_type)):
@@ -140,7 +162,10 @@ def writePlistToString(rootObject, binary=True):
     if not binary:
         rootObject = wrapDataObject(rootObject, binary)
         if six.PY3:
-            return plistlib.writePlistToBytes(rootObject)
+            if hasattr(plistlib, "dumps"):
+                return plistlib.dumps(rootObject)
+            else:
+                return plistlib.writePlistToBytes(rootObject)
         else:
             return plistlib.writePlistToString(rootObject)
     else:
@@ -285,7 +310,7 @@ class PlistReader(object):
         result = 0
         original_offset = self.currentOffset
         data = self.contents[self.currentOffset:self.currentOffset+bytes]
-        result = self.getSizedInteger(data, bytes)
+        result = self.getSizedInteger(data, bytes, as_number=True)
         self.currentOffset = original_offset + bytes
         return result
     
@@ -364,7 +389,8 @@ class PlistReader(object):
     def readUid(self, length):
         return Uid(self.readInteger(length+1))
     
-    def getSizedInteger(self, data, bytes):
+    def getSizedInteger(self, data, bytes, as_number=False):
+        """Numbers of 8 bytes are signed integers when they refer to numbers, but unsigned otherwise."""
         result = 0
         # 1, 2, and 4 byte integers are unsigned
         if bytes == 1:
@@ -374,9 +400,21 @@ class PlistReader(object):
         elif bytes == 4:
             result = unpack('>L', data)[0]
         elif bytes == 8:
-            result = unpack('>q', data)[0]
+            if as_number:
+                result = unpack('>q', data)[0]
+            else:
+                result = unpack('>Q', data)[0]
+        elif bytes <= 16:
+            # Handle odd-sized or integers larger than 8 bytes
+            # Don't naively go over 16 bytes, in order to prevent infinite loops.
+            result = 0
+            if hasattr(int, 'from_bytes'):
+                result = int.from_bytes(data, 'big')
+            else:
+                for byte in data:
+                    result = (result << 8) | unpack('>B', byte)[0]
         else:
-            raise InvalidPlistException("Encountered integer longer than 8 bytes.")
+            raise InvalidPlistException("Encountered integer longer than 16 bytes.")
         return result
 
 class HashableWrapper(object):
@@ -391,6 +429,19 @@ class BoolWrapper(object):
     def __repr__(self):
         return "<BoolWrapper: %s>" % self.value
 
+class FloatWrapper(object):
+    _instances = {}
+    def __new__(klass, value):
+        # Ensure FloatWrapper(x) for a given float x is always the same object
+        wrapper = klass._instances.get(value)
+        if wrapper is None:
+            wrapper = object.__new__(klass)
+            wrapper.value = value
+            klass._instances[value] = wrapper
+        return wrapper
+    def __repr__(self):
+        return "<FloatWrapper: %s>" % self.value
+
 class PlistWriter(object):
     header = six.b('bplist00bybiplist1.0')
     file = None
@@ -467,6 +518,8 @@ class PlistWriter(object):
                 return self.wrappedTrue
             else:
                 return self.wrappedFalse
+        elif isinstance(root, float):
+            return FloatWrapper(root)
         elif isinstance(root, set):
             n = set()
             for value in root:
@@ -522,7 +575,7 @@ class PlistWriter(object):
         elif isinstance(obj, six.integer_types):
             size = self.intSize(obj)
             self.incrementByteCount('intBytes', incr=1+size)
-        elif isinstance(obj, (float)):
+        elif isinstance(obj, FloatWrapper):
             size = self.realSize(obj)
             self.incrementByteCount('realBytes', incr=1+size)
         elif isinstance(obj, datetime.datetime):    
@@ -608,8 +661,8 @@ class PlistWriter(object):
             bytes = self.intSize(obj)
             root = math.log(bytes, 2)
             output += pack('!B', (0b0001 << 4) | int(root))
-            output += self.binaryInt(obj)
-        elif isinstance(obj, float):
+            output += self.binaryInt(obj, as_number=True)
+        elif isinstance(obj, FloatWrapper):
             # just use doubles
             output += pack('!B', (0b0010 << 4) | 3)
             output += self.binaryReal(obj)
@@ -686,10 +739,10 @@ class PlistWriter(object):
     
     def binaryReal(self, obj):
         # just use doubles
-        result = pack('>d', obj)
+        result = pack('>d', obj.value)
         return result
     
-    def binaryInt(self, obj, bytes=None):
+    def binaryInt(self, obj, bytes=None, as_number=False):
         result = six.b('')
         if bytes is None:
             bytes = self.intSize(obj)
@@ -700,9 +753,17 @@ class PlistWriter(object):
         elif bytes == 4:
             result += pack('>L', obj)
         elif bytes == 8:
-            result += pack('>q', obj)
+            if as_number:
+                result += pack('>q', obj)
+            else:
+                result += pack('>Q', obj)
+        elif bytes <= 16:
+            try:
+                result = pack('>Q', 0) + pack('>Q', obj)
+            except struct_error as e:
+                raise InvalidPlistException("Unable to pack integer %d: %s" % (obj, e))
         else:
-            raise InvalidPlistException("Core Foundation can't handle integers with size greater than 8 bytes.")
+            raise InvalidPlistException("Core Foundation can't handle integers with size greater than 16 bytes.")
         return result
     
     def intSize(self, obj):
@@ -719,8 +780,10 @@ class PlistWriter(object):
             return 4
         # SIGNED
         # 0x7FFFFFFFFFFFFFFF is the max.
-        elif obj <= 0x7FFFFFFFFFFFFFFF: # 8 bytes
+        elif obj <= 0x7FFFFFFFFFFFFFFF: # 8 bytes signed
             return 8
+        elif obj <= 0xffffffffffffffff: # 8 bytes unsigned
+            return 16
         else:
             raise InvalidPlistException("Core Foundation can't handle integers with size greater than 8 bytes.")
     
